// Copyright (c) 2016--18, Norman Gray
// All rights reserved.
// 
// Redistribution and use in source and binary forms,
// with or without modification,
// are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// --------
// unittests module, revision 83935ffef7cf, of 2018-08-18 11:40 +0100
// See https://bitbucket.org/nxg/unittests
// --------


// Simple unit testing support
//
// This software is Copyright 2016--18, Norman Gray,
// and is available under the terms of the 2-clause BSD licence.
// Homepage at https://bitbucket.org/nxg/unittests

#ifndef UNITTESTS_H
#define UNITTESTS_H 1

#ifndef ARDUINO
#define ARDUINO 0
#endif

#include <stddef.h>             // for size_t
#include <stdio.h>              // for s(n)printf

#if !ARDUINO
#include <string>
#endif

void set_test_name(const char* name);
#if ARDUINO
const char* get_test_name(void);
#else
std::string get_test_name(void);
#endif

bool get_test_verbose_p(void);
bool set_test_verbose_p(bool);
void assert_equal_(const char* fname, const int line, const void* expected, const void* actual);
#if !ARDUINO
void assert_equal_(const char* fname, const int line, const std::string expected, const std::string actual);
void assert_equal_(const char* fname, const int line, const char* expected, const std::string actual);
#endif
void assert_equal_(const char* fname, const int line, const char* expected, const char* actual);
//void assert_equal_(const char* fname, const int line, int expected, int actual);
void assert_equal_(const char* fname, const int line, const long expected, const long actual);
//void assert_equal_(const char* fname, const int line, double expected, double actual);
void assert_equal_(const char* fname, const int line, const double expected, const double actual, const double tolerance);
void assert_equal_abs_(const char* fname, const int line, const double expected, const double actual, const double tolerance);
void assert_true_(const char* fname, const int line, const int, const char* msg);
void assert_false_(const char* fname, const int line, const int, const char* msg);
void assert_null_(const char* fname, const int line, const void* p, const char* msg);
void assert_not_null_(const char* fname, const int line, const void* p, const char* msg);
void assert_pass(void);
void assert_fail_(const char* fname, const int line, const char* fmt, ...);

// assert_equal_ can have two or three arguments
#define assert_equal(...) assert_equal_(__FILE__, __LINE__, __VA_ARGS__)
#define assert_equal_abs(...) assert_equal_abs_(__FILE__, __LINE__, __VA_ARGS__)
#define assert_true(t) assert_true_(__FILE__, __LINE__, t, #t)
#define assert_false(t) assert_false_(__FILE__, __LINE__, t, #t)
#define assert_null(a) assert_null_(__FILE__, __LINE__, reinterpret_cast<const void*>(a), #a)
#define assert_not_null(a) assert_not_null_(__FILE__, __LINE__, reinterpret_cast<const void*>(a), #a)
#define assert_fail(...) assert_fail_(__FILE__, __LINE__, __VA_ARGS__)

#define SHOULD_FAIL(EXPLANATION, F, ...) do {  \
    int passed_p = 0;                           \
    try {                                       \
        (void) F(__VA_ARGS__);                  \
        passed_p = 1;                           \
    } catch (const char* e) {                   \
        passed_p = 0;                           \
    }                                           \
    if (passed_p) {                             \
        assert_fail(EXPLANATION);               \
    }                                           \
  }                                             \
while(0)

#define SHOULD_PASS(EXPLANATION, F, ...) do {  \
    int passed_p = 0;                           \
    try {                                       \
        (void) F(__VA_ARGS__);                  \
        passed_p = 1;                           \
    } catch (const char* e) {                   \
        passed_p = 0;                           \
    }                                           \
    if (! passed_p) {                             \
        assert_fail(EXPLANATION);               \
    }                                           \
  }                                             \
while(0)


typedef struct {
    const char* name;
    void (*f)(void);
} Testcase;
int run_test_cases(const char* suite_name, const Testcase* cases, const size_t n,
                   int argc, const char*const argv[]);

#endif /* UNITTESTS_H */
